Table to Chart
==============

Convert a table of data to a chart with Morris.js

Migrating from 1.x
------------------

1. Data attributes no longer prefixed by library. Simply use ``data-colors`` instead of ``data-morris-colors`` for example.
1. Data attributes are now only supported lowercase. Convert your markup to match casing.
  - E.g. ``data-behaveLikeLine="true"`` should now be ``data-behavelikeline="true"``
2. Specify your charting library on the table using ``data-chart-library="morris"`` or ``data-chart-library="chartist"``. Default is ``morris``.
3. Charting libraries are unique per charting type. Replace ``drupal_add_library('table_chart', 'tabletochart');`` with one of the following:
  - ``drupal_add_library('table_chart', 'tabletochart_morris');``
  - ``drupal_add_library('table_chart', 'tabletochart_chartist');``
4. The ``data-type`` is deprecated in favour of ``data-chart-type="line"``. Applies to all charting libraries.

Installation
------------

There are three external libraries required. The libraries should be placed in your "libraries" directory.
See the example make file for more information.

Usage
-----

### Automatic
1. Add the class "table-chart" to the wrapping element around your table
2. Ensure the table_chart library is included
    - drupal_add_library('table_chart', 'tabletochart_morris');
3. You have a chart!

You can also select your charting library. ChartistJS and MorrisJS are both supported. Set your data attribute
on your table to ``data-chart-library="chartist"`` to use ChartistJS. MorrisJS is the default.

**N.B. Don't forget to download the appropriate libraries for your desired charts.**

### Views
1. Use the Views to generate your table
2. Add the class "table-chart" to your view
3. Attach the library to the view
    - Easiest way is to use the [library_attach](https://drupal.org/project/library_attach) module.
4. You have a chart!

### Default Settings
The default options are now stored in Drupal.settings.table_chart. They can be altered so set global configuration.

E.g.
- Drupal.settings.table_chart.defaultOptionsMorris
- Drupal.settings.table_chart.defaultOptionsChartist

### Data Attributes

You can control the display of the chart and the columns used using data attributes on your table and columns.
Take any of the following options and preface with 'data-*'.

Ex:

````
<div class="table-chart" data-chart-group="false">
  <table data-chart-type="line">...</table>
</div>
````


Here are the available options:

#### Chart settings

##### Global Options

chart-library
: The library you want to use for your charts. "chartist" and "morris" are supported. Default is "morris".

chart-type
: The type of chart you want to use. Global options include 'bar' and 'line'. Other options available per library.

##### Wrapper options
chart-group
: Boolean to group all tables into a single chart as mutiple data series. Defaults to "false".

##### Chartist Options

chart-type
: The type of chart. Additional types supported include "pie".

areabase
:

chartpadding
:

distributeseries
:

donut
:

donutsolid
:

donutwidth
:

fullwidth
:

height
:

high
:

horizontalbars
:

ignoreemptyvalues
:

labeldirection
:

labeloffset
:

labelposition
:

linesmooth
:

low
:

referencevalue
:

reversedata
:

seriesbardistance
:

showarea
:

showgridbackground
:

showlabel
:

showline
:

showpoint
:

stackbars
:

stackmode
:

startangle
:

total
:

width
:

##### Morris Options

morris-type (DEPRECATED)
: Used 'chart-type' instead.

chart-type
: The type of chart. Additional types supported include "donut" and "area".

colors
: Comma separated list of colors. Hex values (including the # symbol) or a default color (red, green, blue, etc...)

resize
: Boolean value to enable automatic resizing (i.e. responsive charts, requires latest dev version of Morris.js

linewidth
: Specify line width

pointsize
: Specify the point size

pointfillcolors
: Specify point fill colours

pointstrokecolors
: Specify point stroke colours

ymax
: Y max value

ymin
: Y min value

smooth
: Boolean smooth flag

hidehover
: Boolean flag hide hover

hovercallback
: Hover callback function

parsetime
: Boolean parse time

postunits
: Post units

preunits
: Pre units

dateformat
: Date format

xlabels
: X Labels

xlabelformat
: X Lable format

ylabelformat
: Y Lable format

goals
: Goals

goalstrokewidth
: Goal stroke width

goallinecolors
: Goal line colors

events
: Morris events

eventstrokewidth
: Event stroke width

eventlinecolors
: Event line colors

continuousline
: Continuous Line

axes
: Axes

grid
: Grid

gridtextcolor
: Grid text colour

gridtextsize
: Grid text size

gridtextfamily
: Grid text family

gridtextweight
: Gride text weight

fillopacity
: Fill opacity

behavelikeline
: Boolean behave like line

formatter
: Formatter

xkey
: A string containing the name of the attribute that contains date (X) values. (See morris js docs for more details). xKey will default to first column if omitted.

ykeys
: A list of strings containing names of attributes that contain Y values (one for each series of data to be plotted). yKeys will default to second column and beyond if omitted.

labels
: A list of strings containing labels for the data series to be plotted (corresponding to the values in the ykeys option). Labels will be automatically generated if omitted.

stacked
: Set to true to draw bars stacked vertically.

#### Data settings
For more details on each option see https://github.com/lightswitch05/table-to-json

tabletojson-ignorecolumns
: Comma separated list of columns to ignore. Use index values starting at zero (0) (e.g. Ignore the first, third and sixth columns - 0,2,5)

tabletojson-onlycolumns
: Comma separated list of columns to include.

tabletojson-ignorehiddenrows
: Boolean value. Set to "true" or "false" accordingly. (e.g. data-tabletojson-ignoreHiddenRows=true)

tabletojson-headings
: Comma separated list of headings to use. See https://github.com/lightswitch05/table-to-json for more details.