(function ($) {
  Drupal.behaviors.table_chart = {
    attach(context, settings) {
      let count = 0;
      $('.table-chart').each(function () {
        $(this).once(function () {
          const wrapper = $(this);
          let options = {};
          options.chartID = wrapper.attr('id');
          const tables = wrapper.find('table');
          const group_tables = wrapper.data('chart-group');
          if (group_tables === true) {
            // Grab the first mentioned chart library.
            let chartLibrary = tables.data('chart-library');
            const group_options = [];
            group_options.table_data = [];
            // Group all tables together as a collection of series
            // in a single chart.
            tables.each(function (index) {
              const table = $(this);
              options.chart_type = 'line';
              options = prepareTableData(table, options);
       	      group_options.table_data = mergeDataSets(group_options.table_data, options.table_data, count);
              count++;
            });

            options.table_data = group_options.table_data;

            switch (chartLibrary) {
              case 'chartist':
                options = tableToChartChartist($(tables[0]), options, count);
                break;
              default:
              case 'morris':
                chartLibrary = 'morris';
                options = tableToChartMorris($(tables[0]), options, count);
                break;
            }
            // Draw some charts.
            drawChart(wrapper, $(tables[tables.length - 1]), options, chartLibrary);
            count++;
          }
          else {
            // Render each table as it's own chart.
            tables.each(function (index) {
              const table = $(this);
              let chartLibrary = table.data('chart-library');
              options.chart_type = 'line';
              options = prepareTableData(table, options);

              // Check if the charting library and based on that formulate the table from the json object to how we can formulate it.
              switch (chartLibrary) {
                case 'chartist':
                  options = tableToChartChartist(table, options, count);
                  break;
                default:
                case 'morris':
                  chartLibrary = 'morris';
                  options = tableToChartMorris(table, count);
                  break;
              }

              // Draw some charts.
              drawChart(wrapper, table, options, chartLibrary);
              count++;
            });
          }
        });
      });
    },
  };

  /**
   * Generate a unique object key.
   *
   * @return string
   */
  function generateKeyName(data, keyName, count) {
    if (keyName in data) {
      return keyName + count;
    }

    return keyName;
  }

  /**
   * Merge table data sets together.
   *
   * Adds values to existing keys.
   * Adds new keys with new values.
   * Padds entries with null values to complete series.
   * @return array
   */
  function mergeDataSets(data, new_data, count) {
    if (data === undefined || data.length == 0) {
      return new_data;
    }

    // Store the initial size.
    const initialSize = data.length;
    const keyName = Object.keys(data[0])[0];

    // Add to existing key values.
    for (var x = 0; x < data.length; x++) {
      var found = false;
      for (var y = 0; y < new_data.length; y++) {
        // If the data key match, append all the values.
        if (data[x][keyName] == new_data[y][keyName]) {
          found = true;
          $.each(new_data[y], (key, value) => {
            if (key != keyName) {
              data[x][generateKeyName(data[x], key, count)] = value;
            }
          });
        }
      }
      if (!found) {
        // Add existing keys with blank values.
        $.each(new_data[0], (key, value) => {
          if (key != keyName) {
            data[x][generateKeyName(data[x], key, count)] = null;
          }
        });
      }
    }

    // Add missing key/value pairs.
    for (var y = 0; y < new_data.length; y++) {
      var found = false;
      for (var x = 0; x < data.length; x++) {
        // Scan for the key name value. Append if it doesn't exist.
        if (data[x][keyName] == new_data[y][keyName]) {
          found = true;
        }
      }
      if (!found) {
        new_object = {};
        new_object[keyName] = new_data[y][keyName];

        // Add existing keys with blank values.
        $.each(data[0], (key, value) => {
          if (key != keyName) {
            new_object[key] = null;
          }
        });

        // Add new values.
        $.each(new_data[y], (key, value) => {
          if (key != keyName) {
            new_object[generateKeyName(data[0], key, count)] = value;
          }
        });

        // Append to array.
        // @todo sort the results.
        data.push(new_object);
      }
    }
    // @todo

    // Scan through all existing keys. Add values to them.
    // Take remaining keys and pad with empty data slots.
    // Sort by array keys.
    if (data.length != initialSize) {
      data = sortDataSets(data, keyName);
    }
    return data;
  }

  /**
   * Sort merged datasets.
   *
   * @return array
   *  Returns sorted dataset.
   */
  function sortDataSets(data, keyname) {
    // Use a bubblesort algorithm.
    // @see http://blog.benoitvallon.com/sorting-algorithms-in-javascript/sorting-algorithms-in-javascript-all-the-code/
    let swapped;
    do {
      swapped = false;
      for (let i = 0; i < data.length; i++) {
        if (data[i] && data[i + 1] && data[i][keyname] > data[i + 1][keyname]) {
          swap(data, i, i + 1);
          swapped = true;
        }
      }
    } while (swapped);
    return data;
  }

  // swap function helper
  function swap(array, i, j) {
    const temp = array[i];
    array[i] = array[j];
    array[j] = temp;
  }

  // Chartist data formulate data.
  function tableToChartChartist(table, options, count) {
    const headings = [];
    let data_labels = [];
    const data_series = [];

    options.element = `chartist-chart-${count}`;
    options = generateOptionsList(table, options, 'chartist');

    // Tranpose the data to load into labels/series.
    options.table_data = transpose(options.table_data);

    // Grab the list of labels first.
    data_labels = options.table_data[Object.keys(options.table_data)[0]];

    // Based on the number of labels, build the series arrays.
    for (let x = 1; x < Object.keys(options.table_data).length; x++) {
      data_series.push(options.table_data[Object.keys(options.table_data)[x]]);
    }

    // Create the chartis json object to create the markup.
    const data = {
      labels: data_labels,
      series: data_series,
    };

    // Set the table data now that it's ready for chartist.
    options.table_data = data;

    // Set axis-specific configurations.
    options.axisX.labelInterpolationFnc = (value, index, labels) => {
      if (index % Math.ceil(labels.length / ($('.table-chart').width() / 82)) === 0 /* || index === labels.length - 1 */) {
        return labels[index];
      }
      return null;
    };

    // Draw the chart.
    return options;
  }

  // Morris js data formulating.
  function tableToChartMorris(table, options, count) {
    // Chart settings.
    var options = [];
    const optKeys = Drupal.settings.table_chart.defaultOptionsMorris;

    options = generateOptionsList(table, options, 'morris');
    options.element = `morris-chart-${count}`;

    // Get the list of keys
    for (const i in options.table_data) {
      for (const j in options.table_data[i]) {
        options.keys.push(j);
      }
      break;
    }

    // Draw the chart with the given options
    return options;
  }

  /**
   * Generate the chart with the given defaults
   */
  function drawChart(wrapper, table, options, chartLibrary) {
    // Add the data display toggle.
    groupChart = wrapper.data('chart-group');

    const targetId = `source-${options.element}`;
    let targetSelector = `#${targetId}`;
    const hideTarget = '';
    if (groupChart === true) {
      wrapper.attr('id', targetId);
      targetSelector += ' table';
    }
    else {
      table.addClass('element-invisible');
      table.attr('id', targetId);
    }

    // Hide the target element.
    $(targetSelector).addClass('element-invisible');

    table.after(`<div id="${options.element}" class="${chartLibrary}-chart"></div> <a href="#" id="toggle-${options.element}" class="button toggle-table" data-wrapper-target="${targetSelector}" title="${Drupal.t('Toggle Data Table')}">${Drupal.t('Show data')}</a>`);

    $(`#toggle-${options.element}`).click(function (event) {
      $($(this).data('wrapper-target')).toggleClass('element-invisible');

      if ($($(this).data('wrapper-target')).hasClass('element-invisible')) {
        $(this).text(Drupal.t('Show Data'));
      }
      else {
        $(this).text(Drupal.t('Hide Data'));
      }
      event.preventDefault();
    });

    // Render the right chart.
    switch (chartLibrary) {
      default:
      case 'morris':
        drawChartMorris(options);
        break;
      case 'chartist':
        drawChartChartist(options);
        break;
    }
  }

  /**
   * Draw chartist charts.
   *
   * @param {Array} options
   *   Chart options.
   */
  function drawChartChartist(options) {
    switch (options.chart_type) {
      case 'bar':
        Drupal.settings.table_chart[options.chartID] = new Chartist.Bar(`#${options.element}`, options.table_data, options);
        break;
      default:
      case 'line':
        Drupal.settings.table_chart[options.chartID] = new Chartist.Line(`#${options.element}`, options.table_data, options);
        break;
      case 'pie':
        // The pie data is formulated differently.
        var pie_data = {
          series: options.table_data.data_series[0],
        };
        Drupal.settings.table_chart[options.chartID] = new Chartist.Pie(`#${options.element}`, pie_data, options);
        break;
    }
  }

  /**
   * Draw morris charts.
   *
   * @param {Array} options
   *   Chart options.
   */
  function drawChartMorris(options) {
    // Set global settings
    const settings = {
      element: options.element,
      data: options.table_data,
      resize: options.resize,
    };

    // Set per-type settings
    switch (options.chart_type) {
      case 'bar':
        // Add optional settings
        if (options.colors !== null && options.colors != undefined) {
          settings.barColors = options.colors;
        }
        if (options.stacked) {
          settings.stacked = options.stacked;
        }
        else {
          settings.stacked = settings.stacked;
        }
        if (options.hideHover !== null) {
          settings.hideHover = options.hideHover;
        }
        if (options.hoverCallback !== null) {
          settings.hoverCallback = options.hoverCallback;
        }
        if (options.axes !== null) {
          settings.axes = options.axes;
        }
        if (options.grid !== null) {
          settings.grid = options.grid;
        }
        if (options.gridTextColor !== null) {
          settings.gridTextColor = options.gridTextColor;
        }
        if (options.gridTextSize !== null) {
          settings.gridTextSize = options.gridTextSize;
        }
        if (options.gridTextFamily !== null) {
          settings.gridTextFamily = options.gridTextFamily;
        }
        if (options.gridTextWeight !== null) {
          settings.gridTextWeight = options.gridTextWeight;
        }
        if (options.hideHover !== null) {
          settings.hideHover = options.hideHover;
        }

        // Add required settings
        if (options.xkey != null) {
          settings.xkey = options.xkey;
        }
        else {
          settings.xkey = options.keys.shift();
        }

        if (options.ykeys.length > 0) {
          settings.ykeys = options.ykeys;
        }
        else {
          settings.ykeys = options.keys;
        }

        if (options.labels != null) {
          settings.labels = options.labels;
        }
        else {
          settings.labels = settings.ykeys;
        }

        new Morris.Bar(settings);
        break;
      case 'donut':
        if (options.colors !== null && options.colors != undefined) {
          settings.colors = options.colors;
        }
        if (options.formatter !== null) {
          settings.formatter = options.formatter;
        }
        new Morris.Donut(settings);

        break;
      case 'area':
        // Add optional settings
        if (options.behaveLikeLine !== null) {
          settings.behaveLikeLine = options.behaveLikeLine;
        }

      // Area and Line share all the same configuration except the one option
      // So we fall into the default case and check at the end which type to
      // generate.
      case 'line':
      default:
        // Add optional settings

        if (options.colors !== null && options.colors != undefined) {
          settings.lineColors = options.colors;
        }

        // Add required settings
        if (options.xkey != null) {
          settings.xkey = options.xkey;
        }
        else {
          settings.xkey = options.keys.shift();
        }

        if (options.ykeys.length > 0) {
          settings.ykeys = options.ykeys;
        }
        else {
          settings.ykeys = options.keys;
        }

        if (options.labels != null) {
          settings.labels = options.labels;
        }
        else {
          settings.labels = settings.ykeys;
        }

        // @todo adjust line settings
        if (options.type == 'area') {
          new Morris.Area(settings);
        }
        else {
          new Morris.Line(settings);
        }
    }
  }
  /**
   * Matrix transpose function.
   *
   * @param {Array} data
   * @return array
   */
  function transpose(data) {
    const result = {};
    for (const row of data) {
      for (const [key, value] of Object.entries(row)) {
        result[key] = result[key] || [];
        result[key].push(value);
      }
    }

    return result;
  }

  /**
   * Convert table into json array.
   */
  function prepareTableData(table, options = []) {
    // Ignoring the first column (i.e. 0) has some odd behavior. Zero (0)
    // being misinterpreted as the emtpy array value. So we treat this
    // case separately.
    if (table.data('tabletojson-ignorecolumns') == 0) {
      options.ignoreColumns = [0];
    }
    else {
      options.ignoreColumns = table.data('tabletojson-ignorecolumns') ? String(table.data('tabletojson-ignorecolumns')).split(',') : [];
    }
    // Same issue as above
    if (table.data('tabletojson-onlycolumns') == 0) {
      options.onlyColumns = [0];
    }
    else {
      options.onlyColumns = table.data('tabletojson-onlycolumns') ? table.data('tabletojson-onlycolumns').split(',') : null;
    }
    options.ignoreHiddenRows = table.data('tabletojson-ignorehiddenrows') ? table.data('tabletojson-ignorehiddenrows') == true : true;
    options.headings = table.data('tabletojson-headings') ? table.data('tabletojson-headings').split(',') : null;
    options.keys = [];
    // Data preperation

    // Ensure all the values are true integers and not string numbers
    options.ignoreColumns = options.ignoreColumns.map(x => parseInt(x, 10));

    // Read the table data with the given configuration
    options.table_data = table.tableToJSON({
      ignoreColumns: options.ignoreColumns,
      onlyColumns: options.onlyColumns,
      ignoreHiddenRows: options.ignoreHiddenRows,
      headings: options.headings,
    });

    // Store the table headings for later use.
    options.headings = Object.keys(options.table_data[0]);

    return options;
  }

  /**
   * Generate the list of options.
   *
   * Parses the data attributes and sets their type properly.
   */
  function generateOptionsList(table, options, charting_type) {
    let optKeys = [];
    switch (charting_type) {
      default:
      case 'morris':
        optKeys = Drupal.settings.table_chart.defaultOptionsMorris;
        break;
      case 'chartist':
        optKeys = Drupal.settings.table_chart.defaultOptionsChartist;
        optKeys = optKeys.concat(Drupal.settings[options.chartID].defaultOptionsChartist);
        break;
    }

    // Extract settings from data attributes if provided.
    $.each(optKeys, (idx, optData) => {
      const key = optData[0];


      const type = optData[1];


      let defaultValue = optData[2];

      // Set to undefined if the string undefined is passed.
      if (defaultValue == 'undefined') {
        defaultValue = undefined;
      }

      // Check attribute keys. Only lowercase values.
      let attrKey = null;
      if (table.data(key.toLowerCase())) {
        attrKey = key.toLowerCase();
      }

      switch (type) {
        default:
        case 'str':
          options[key] = attrKey != null ? table.data(attrKey) : defaultValue;
          break;
        case 'bool':
          options[key] = attrKey != null ? table.data(attrKey) : defaultValue;
          break;
        case 'int':
          options[key] = attrKey != null ? parseInt(table.data(attrKey), 10) : defaultValue;
          break;
        case 'float':
          options[key] = attrKey != null ? parseFloat(table.data(attrKey)) : defaultValue;
          break;
        case 'array':
          options[key] = attrKey != null ? table.data(attrKey).split(',') : defaultValue;
          break;
        case 'json':
          options[key] = attrKey != null ? table.data(attrKey) : defaultValue;
          break;
        case 'plugin':
          if (attrKey != null) {
            table.data(attrKey).forEach((element) => {
              // function we want to run
              var fnstring = '';
              var fnargs = {};

              if (Array.isArray(element)) {
                // function we want to run
                var fnstring = element[0];
                var fnargs = element[1];
              }
              else {
                // function we want to run
                var fnstring = element;
              }

              // find object
              const fn = window[fnstring];

              // is object a function?
              if (typeof fn === 'function') {
                if (Array.isArray(options[key])) {
                  options[key].push(fn(fnargs));
                }
                else {
                  options[key] = [fn(fnargs)];
                }
              }
            });
          }
          else {
            defaultValue.forEach((element) => {
              var fnstring = '';
              var fnargs = {};

              if (Array.isArray(element)) {
                // function we want to run
                var fnstring = element[0];
                var fnargs = element[1];
              }
              else {
                // function we want to run
                var fnstring = element;
              }

              // find object
              const fn = window[fnstring];

              // is object a function?
              if (typeof fn === 'function') {
                if (Array.isArray(options[key])) {
                  options[key].push(fn(fnargs));
                }
                else {
                  options[key] = [fn(fnargs)];
                }
              }
            });
          }

          break;
      }
    });

    return options;
  }
}(jQuery));
